function fastica_gui(mixedsig, InitialGuess)
//Fast Independent Component Analysis, the Graphical User Interface
// Calling Sequence
//  fastica_gui()
//  fastica_gui(mixedsig)
//  fastica_gui(mixedsig, InitialGuess)
// Description
// fastica_gui gives a graphical user interface for performing independent
// component analysis by the FastICA package. No arguments are
// necessary in the function call.
//
// Optional arguments can be given in the form:
// fastica_gui(mixedsig, initialGuess) where the matrix mixedsig contains the
// multidimensional signals as row vectors, and initialGuess gives the
// initial value for the mixing matrix used in the algorithm.
//
// fastica_gui uses the fixed-point algorithm developed by Aapo Hyvarinen,
// see http://www.cis.hut.fi/projects/ica/fastica/. The Matlab package
// was programmed by Hugo Gavert, Jarmo Hurri, Jaakko Sarela, and Aapo
// Hyvarinen.
//
//
//   See also 
//   fastica
//   Authors
// H. Nahrstaedt - 2011 
//   (C) 1996-2005 by Hugo Gvert, Jarmo Hurri, Jaakko Srel, and Aapo Hyvrinen. 

// @(#)$Id: fasticag.m,v 1.5 2005/10/19 13:05:34 jarmo Exp $


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global values
 [nargout,nargin]=argn(0);
// Handle to this main figure
global hf_FastICA_MAIN;

// Check to see if GUI is already running
// Can't have more than one copy - otherwise the global
// variables and handles can get mixed up.
// if ~isempty(hf_FastICA_MAIN)
//   error('FastICA GUI already running!');
// end


// Handles to other controls in this main window
global ht_FastICA_mixedStatus;
global ht_FastICA_dim;
global ht_FastICA_numOfSamp;
global ht_FastICA_newDim;
global ht_FastICA_whiteStatus;
global ht_FastICA_icaStatus;
global hpm_FastICA_approach;
global he_FastICA_numOfIC;
global hpm_FastICA_g;
global hpm_FastICA_stabilizat;

// These global variables are used to store all the values
// I used to use the 'UserData' field of components, but
// that got too complex, so I decided to put everything
// in global variables
global g_FastICA_mixedsig;
global g_FastICA_pca_D;
global g_FastICA_pca_E;
global g_FastICA_white_sig;
global g_FastICA_white_wm;
global g_FastICA_white_dwm;
global g_FastICA_ica_sig;
global g_FastICA_ica_A;
global g_FastICA_ica_W;
global g_FastICA_initGuess;
global g_FastICA_approach;
global g_FastICA_numOfIC;
global g_FastICA_g;
global g_FastICA_finetune;
global g_FastICA_a1;
global g_FastICA_a2;
global g_FastICA_myy;
global g_FastICA_stabilization;
global g_FastICA_epsilon;
global g_FastICA_maxNumIte;
global g_FastICA_maxFinetune;
global g_FastICA_sampleSize;
global g_FastICA_initState;
global g_FastICA_displayMo;
global g_FastICA_displayIn;
global g_FastICA_verbose;

// initial values for them:
// All the initial values are set here - even for
// variables that are not used in this file

if nargin < 2
  g_FastICA_initGuess = 1;
  // The user didn't enter initial guess so we default
  // back to random initial state.
  g_FastICA_initState = 1;  // see below for string values
else
  g_FastICA_initGuess = InitialGuess;
  // If initial guess was entered, then the user probably
  // wan't to use it, eh?
  g_FastICA_initState = 2;  // see below for string values
end

if nargin < 1
  g_FastICA_mixedsig = [];
else
  g_FastICA_mixedsig = mixedsig; // We'll remove mean
end                              // the first time we
                                 // use this.

// Global variable for stopping the ICA calculations
global g_FastICA_interrupt;

g_FastICA_pca_D = [];
g_FastICA_pca_E = [];
g_FastICA_white_sig = [];
g_FastICA_white_wm = [];
g_FastICA_white_dwm = [];
g_FastICA_ica_sig = [];
g_FastICA_ica_A = [];
g_FastICA_ica_W = [];
g_FastICA_approach = 1;   // see below for string values
g_FastICA_numOfIC = 0;
g_FastICA_g = 1;          // see below for string values
g_FastICA_finetune = 5;   // see below for string values
g_FastICA_a1 = 1;
g_FastICA_a2 = 1;
g_FastICA_myy = 1;
g_FastICA_stabilization = 2; // see below for string values
g_FastICA_epsilon = 0.0001;
g_FastICA_maxNumIte = 1000;
g_FastICA_maxFinetune = 100;
g_FastICA_sampleSize = 1;
g_FastICA_displayMo = 1;  // see below for string values
g_FastICA_displayIn = 1;
g_FastICA_verbose = 1;    // see below for string values

// These are regarded as constants and are used to store
// the strings for the popup menus the current value is
// seen in the variables above
// D - refers to strings that are displayed
// V - refers to string values that are used in FPICA
global c_FastICA_appr_strD;
global c_FastICA_appr_strV;
global c_FastICA_g1_strD;
global c_FastICA_g1_strV;
global c_FastICA_g2_strD;
global c_FastICA_g2_strV;
global c_FastICA_finetune_strD;
global c_FastICA_finetune_strV;
global c_FastICA_stabili_strD;
global c_FastICA_stabili_strV;
global c_FastICA_iSta_strD;
global c_FastICA_iSta_strV;
global c_FastICA_dMod_strD;
global c_FastICA_dMod_strV;
global c_FastICA_verb_strD;
global c_FastICA_verb_strV;

// All the values for these are set here - even for
// variables that are not used in this file

c_FastICA_appr_strD = 'deflation|symmetric';
c_FastICA_appr_strV = ['defl';'symm'];
// The 'g1' and 'g2' below correspond to the values of approach (1 or 2)
// Deflation and Symmetric used to have a bit different selection
// of available nonlinearities.
c_FastICA_g1_strD = 'pow3|tanh|gauss|skew';
c_FastICA_g1_strV = ['pow3';'tanh';'gaus';'skew'];
c_FastICA_g2_strD = 'pow3|tanh|gauss|skew';
c_FastICA_g2_strV = ['pow3';'tanh';'gaus';'skew'];
c_FastICA_finetune_strD = 'pow3|tanh|gauss|skew|off';
c_FastICA_finetune_strV = ['pow3';'tanh';'gaus';'skew';'off '];
c_FastICA_stabili_strD = 'on|off';
c_FastICA_stabili_strV = ['on ';'off'];
c_FastICA_iSta_strD = 'random|guess';
c_FastICA_iSta_strV = ['rand ';'guess'];
c_FastICA_dMod_strD = 'signals|basis|filters|off';
c_FastICA_dMod_strV = ['signals';'basis  ';'filters';'off    '];
c_FastICA_verb_strD = 'on|off';
c_FastICA_verb_strV = ['on ';'off'];


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Configuration options
FIGURENAME = 'FastICA';
FIGURETAG = 'f_FastICA';
//SCREENSIZE = get(0,'ScreenSize');
//FIGURESIZE = [round(0.1*SCREENSIZE(3)) (SCREENSIZE(4)-round(0.1*SCREENSIZE(4))-370) 530 370];

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create the figure
// a = figure('Color',[0.8 0.8 0.8], ...
// 	   'PaperType','a4letter', ...
// 	   'Name', FIGURENAME, ...
// 	   'NumberTitle', 'off', ...
// 	   'Tag', FIGURETAG, ...
// 	   'Position', FIGURESIZE, ...
// 	   'MenuBar', 'none');
a = scf();
a.figure_name=FIGURENAME;
a.tag=FIGURETAG;
FIGURESIZE=[a.figure_position a.figure_size];
FIGURESIZE=[a.figure_position 750, 550];
a.figure_size=[800 700];
// Resizing has to be denied after the window has been created -
// otherwise the window shows only as a tiny window in Windows XP.
//set (a, 'Resize', 'off');

hf_FastICA_MAIN = a;

//set(hf_FastICA_MAIN, 'HandleVisibility', 'callback');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// From here on it get's ugly as I have not had time to clean it up


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create the frames
pos_l=2;
pos_w=FIGURESIZE(3)-4;
pos_h=FIGURESIZE(4)-4;
pos_t=FIGURESIZE(4)-2-pos_h;
h_f_background = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_background');

pos_l=4;
pos_w=400;
pos_h=106;
pos_t=FIGURESIZE(4)-4-pos_h;
h_f_mixed = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_mixed');

pos_h=90;
pos_t=FIGURESIZE(4)-(106+4+2)-pos_h;
h_f_white = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_white');

pos_h=pos_t - 4 - 2;
pos_t=4;
h_f_ica = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_ica');

pos_w=120;
pos_l=FIGURESIZE(3)-(pos_w+2+2);
pos_h=FIGURESIZE(4)-2*4;
pos_t=FIGURESIZE(4)-(4)-pos_h;
h_f_side = uicontrol('Parent',a, ...
  'BackgroundColor',[0.5 0.5 0.5], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_side');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_mixed
bgc = get(h_f_mixed, 'BackgroundColor');

pos_vspace = 6;
pos_hspace = 6;

pos_frame=get(h_f_mixed, 'Position');
pos_l = pos_frame(1) + 6;
pos_h = 16;
pos_t = pos_frame(2) + pos_frame(4) - pos_h - 6;
pos_w = 150;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Mixed signals:', ...
  'FontWeight', 'bold', ...
  'Style','text', ...
  'Tag','t_mixed');

pos_l = pos_l + pos_w;
pos_w = 120;
ht_FastICA_mixedStatus = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Not loaded yet', ...
  'Style','text', ...
  'Tag','t_mixedstatus');

// Vhn vli
pos_t = pos_t - 8;

pos_l = pos_frame(1) + 6;
pos_t = pos_t - pos_h;
pos_w = 150;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Number of signals:', ...
  'Style','text', ...
  'Tag','t_2');

pos_l = pos_l + pos_w;
pos_w = 50;
ht_FastICA_dim = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','', ...
  'Style','text', ...
  'Tag','t_dim');

pos_l = pos_frame(1) + 6;
pos_t = pos_t - pos_h;
pos_w = 150;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Number of samples:', ...
  'Style','text', ...
  'Tag','t_3');

pos_l = pos_l + pos_w;
pos_w = 50;
ht_FastICA_numOfSamp = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','', ...
  'Style','text', ...
  'Tag','t_numOfSamp');


// Buttons
pos_l = pos_frame(1) + pos_hspace;
pos_w = 110;
pos_h = 30;
pos_t = pos_frame(2) + pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb Transpose', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Transpose', ...
  'Tag','b_Transpose');

  //'Interruptible', 'off', ...

pos_w = 130;
pos_l = pos_frame(1) + pos_frame(3) - pos_hspace - pos_w;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb ShowMixed', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Plot data', ...
  'Tag','b_ShowMixed');

// 'Interruptible', 'off', ...

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_white
pos_frame=get(h_f_white, 'Position');
pos_l = pos_frame(1) + 6;
pos_h = 16;
pos_t = pos_frame(2) + pos_frame(4) - pos_h - 6;
pos_w = 150;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Dimension control:', ...
  'FontWeight', 'bold', ...
  'Style','text', ...
  'Tag','t_white');

pos_l = pos_l + pos_w;
pos_w = 120;
ht_FastICA_whiteStatus = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','', ...
  'Style','text', ...
  'Tag','t_whiteStatus');

// Vhn vli
pos_t = pos_t - 8;

pos_l = pos_frame(1) + 6;
pos_t = pos_t - pos_h;
pos_w = 150;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Reduced dimension:', ...
  'Style','text', ...
  'Tag','t_4');

pos_l = pos_l + pos_w;
pos_w = 50;
ht_FastICA_newDim = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','', ...
  'Style','text', ...
  'Tag','t_newDim');


// buttons

pos_l = pos_frame(1) + pos_hspace;
pos_w = 110;
pos_h = 30;
pos_t = pos_frame(2) + pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb DoPCA', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Reduce dim.', ...
  'Tag','b_DoPCA');

  //'Interruptible', 'off', ...

pos_l = pos_l + pos_w + pos_hspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb OrigDim', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Original dim.', ...
  'Tag','b_OrigDim');

  //'Interruptible', 'off', ...

pos_w = 130;
pos_h = 30;
pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w;
pos_t = pos_frame(2) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb ShowWhite', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Plot whitened', ...
  'Tag','b_ShowWhite');

  //'Interruptible', 'off', ...

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_ica
pos_frame=get(h_f_ica, 'Position');
pos_l = pos_frame(1) + 6;
pos_h = 20;
pos_t = pos_frame(2) + pos_frame(4) - pos_h - 6;
pos_w = 150;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Fixed point ICA:', ...
  'FontWeight', 'bold', ...
  'Style','text', ...
  'Tag','t_white');

pos_l = pos_l + pos_w;
pos_w = 120;
ht_FastICA_icaStatus = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Not yet done', ...
  'Style','text', ...
  'Tag','t_icaStatus');

// Vhn vli
pos_t = pos_t - 8;

//pos_l = pos_frame(1) + 6;
pos_l = pos_frame(1) + 6 + 150;
pos_t = pos_t - pos_h;
//pos_w = 260;
pos_w = 120;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Approach:', ...
  'Style','text', ...
  'Tag','t_5');

pos_w = 100;
//pos_t = pos_t - 4;
pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w;
hpm_FastICA_approach = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'Callback','fastica_gui_cb ChangeApproach', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String',c_FastICA_appr_strD, ...
  'Style','popupmenu', ...
  'Tag','pm_approach', ...
  'Value',g_FastICA_approach);

//pos_t = pos_t - 4;
//pos_l = pos_frame(1) + 6;
pos_l = pos_frame(1) + 6 + 150;
pos_t = pos_t - pos_h;
//pos_w = 260;
pos_w = 120;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Number of ICs:', ...
  'Style','text', ...
  'Tag','t_6');

pos_w = 100;
pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w;
he_FastICA_numOfIC = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'Callback','fastica_gui_cb ChangeNumOfIC', ...
  'HorizontalAlignment','right', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','0', ...
  'Style','edit', ...
  'Tag','e_numOfIC');

//pos_t = pos_t - 4;
//pos_l = pos_frame(1) + 6;
pos_l = pos_frame(1) + 6 + 150;
pos_t = pos_t - pos_h;
//pos_w = 260;
pos_w = 120;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Nonlinearity (g):', ...
  'Style','text', ...
  'Tag','t_71');

//pos_t = pos_t - 4;
pos_w = 100;
pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w;
hpm_FastICA_g = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'Callback','fastica_gui_cb ChangeG', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String',c_FastICA_g1_strD, ...
  'Style','popupmenu', ...
  'Tag','pm_g', ...
  'Value',g_FastICA_g);

//pos_t = pos_t - 4;
//pos_l = pos_frame(1) + 6;
pos_l = pos_frame(1) + 6 + 150;
pos_t = pos_t - pos_h;
//pos_w = 260;
pos_w = 120;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Stabilization:', ...
  'Style','text', ...
  'Tag','t_71a');

//pos_t = pos_t - 4;
pos_w = 100;
pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w;
hpm_FastICA_stabilizat = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'Callback','fastica_gui_cb ChangeStab', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String',c_FastICA_stabili_strD, ...
  'Style','popupmenu', ...
  'Tag','pm_stab', ...
  'Value',g_FastICA_stabilization);



pos_l = pos_frame(1) + pos_vspace;
pos_w = 110;
pos_h = 30;
pos_t = pos_frame(2) + pos_hspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_cb AdvOpt', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Adv. options >>', ...
  'Tag','b_advOpt');

pos_w = 130;
pos_h = 30;
pos_l = pos_frame(1) + pos_frame(3) - pos_vspace - pos_w;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb ShowICASig', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Plot ICs', ...
  'Tag','b_ShowICASig');

//'Interruptible', 'on', ...


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_side
pos_vspace = 6;
pos_hspace = 10;
pos_temp=get(h_f_side, 'Position');
pos_l=pos_temp(1)+pos_hspace;
pos_w=100;
pos_h=30;
pos_t=pos_temp(2)+pos_temp(4)-pos_vspace-pos_h;

b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb LoadData', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Load data', ...
  'Tag','b_LoadData');

  //'Interruptible', 'off', ...

pos_t=pos_t-pos_h-pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb DoFPICA', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Do ICA', ...
  'Tag','b_DoFPICA');

  //'Interruptible', 'on', ...

pos_t=pos_t-pos_h-pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb(''SaveData'')', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Save results', ...
  'Tag','b_SaveData');

  //'Interruptible', 'off', ...

pos_t=pos_t-pos_h-pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb(''Quit'')', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Quit', ...
  'Tag','b_Quit');

pos_t=pos_t-pos_h-pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb Interrupt', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Interrupt', ...
  'Visible','off', ...
  'Tag','b_Interrupt');

pos_t = pos_frame(2) + pos_vspace + pos_h + pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb About', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','About...', ...
  'Tag','b_About');

pos_t = pos_frame(2) + pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_cb Help', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Help', ...
  'Tag','b_Help');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Do rest of the initialization...
  fastica_gui_cb("InitAll");

endfunction












function fastica_gui_cb(action)
//
// This file is used by FASTICAG

// This file holds the callbacks to the main window

// @(#)$Id: gui_cb.m,v 1.5 2003/09/10 10:33:41 jarmo Exp $

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables

// Handle to the main figure
global hf_FastICA_MAIN;

// Handles for needed controls in main figure;
global ht_FastICA_mixedStatus;
global ht_FastICA_dim;
global ht_FastICA_numOfSamp;
global ht_FastICA_newDim;
global ht_FastICA_whiteStatus;
global ht_FastICA_icaStatus;
global hpm_FastICA_approach;
global he_FastICA_numOfIC;
global hpm_FastICA_g;
global hpm_FastICA_stabilizat;

// Main values are stored here
global g_FastICA_mixedsig;
global g_FastICA_mixedmean;
global g_FastICA_pca_D;
global g_FastICA_pca_E;
global g_FastICA_white_sig;
global g_FastICA_white_wm;
global g_FastICA_white_dwm;
global g_FastICA_ica_sig;
global g_FastICA_ica_A;
global g_FastICA_ica_W;
global g_FastICA_initGuess;
global g_FastICA_approach;
global g_FastICA_numOfIC;
global g_FastICA_g;
global g_FastICA_finetune;
global g_FastICA_a1;
global g_FastICA_a2;
global g_FastICA_myy;
global g_FastICA_stabilization;
global g_FastICA_epsilon;
global g_FastICA_maxNumIte;
global g_FastICA_maxFinetune;
global g_FastICA_sampleSize;
global g_FastICA_initState;
global g_FastICA_displayMo;
global g_FastICA_displayIn;
global g_FastICA_verbose;

// String values are here
global c_FastICA_appr_strV;
global c_FastICA_g1_strD;
global c_FastICA_g1_strV;
global c_FastICA_g2_strD;
global c_FastICA_g2_strV;
global c_FastICA_finetune_strD;
global c_FastICA_finetune_strV;
global c_FastICA_stabili_strV;
global c_FastICA_iSta_strV;
global c_FastICA_dMod_strV;
global c_FastICA_verb_strV;

// What is the load type of load dialog
global g_FastICA_loadType;

// Global variable for stopping the ICA calculations
global g_FastICA_interrupt;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// What ever we do, it will take some time... not much, but some :-)
//watchonInFigure = watchon;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
select action
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'InitAll' then

 // If the data is already loaded, then get the information from data
 // and show to the user (also set g_FastICA_numOfIC)
 if ~isempty(g_FastICA_mixedsig)
   set(ht_FastICA_mixedStatus, 'String', '');
   [dim, numofsamp] = size(g_FastICA_mixedsig);
   set(ht_FastICA_dim, 'String', string(dim));
   set(ht_FastICA_numOfSamp, 'String', string(numofsamp));
   set(ht_FastICA_newDim, 'String', string(dim));
   set(he_FastICA_numOfIC, 'String', string(dim));
   g_FastICA_numOfIC = dim;
   g_FastICA_mixedmean = [];
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'LoadData' then

 handle = findobj('Tag','f_FastICALoad');  // Check if the window is already
 if isempty(handle)                        // open. If not then open it.
   pos = get(hf_FastICA_MAIN, 'Position');
   // Based on the feedback obtained from some users, it seems
   // that at least in some systems, pos can sometimes be empty. A
   // similar check is done a few lines below.
   if ~isempty (pos),
     fastica_gui_l(pos(1), pos(2),'LoadData');
   else
     fastica_gui_l (0, 0,'LoadData');
   end
 else
   if (g_FastICA_loadType== 'data')   // Check if it was the same load
     figure(handle);                       // window. If it wasn't then
   else                                    // close the other window first
     close(handle);                        // and then open the load window
     printf('''Load initial guess'' -dialog closed!\n');
     pos = get(hf_FastICA_MAIN, 'Position');
     if ~isempty (pos),
       fastica_gui_l(pos(1), pos(2),'LoadData');
     else
       fastica_gui_l (0, 0,'LoadData');
     end
   end
 end
 
 // gui_cb NewData; - is called from the load function if not canceled...

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'NewData' then

 // New data is loaded or the old data changed. We need to find out
 // somethings about the new data... and do some other stuff also...
 [dim, numofsamp] = size(g_FastICA_mixedsig);
 set(ht_FastICA_dim, 'String', string(dim));
 set(ht_FastICA_newDim, 'String', string(dim));
 set(ht_FastICA_numOfSamp, 'String', string(numofsamp));
 set(he_FastICA_numOfIC, 'String', string(dim));
 
 g_FastICA_numOfIC = dim;    // Default for numOfIC = the new dimension
			     // PCA needs to be calculated again.
 g_FastICA_pca_E = [];       // We use this to check if PCA is calculated
 fastica_gui_cb("NullWhite");           // Whitening needs to be done again also
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'NullWhite' then

 // Whitening needs to done again next time it's needed
 g_FastICA_white_sig = [];   // We use this to check if whitening is calculated
 fastica_gui_cb("NullICA");             // The FPICA must be calculated again

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'NullICA' then

 // If IC's are needed they have to bee calculated again
 g_FastICA_ica_sig = [];     // We use this to check if FPICA is calculated
 set(ht_FastICA_icaStatus,'String','Not yet done');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Transpose' then

 if isempty(g_FastICA_mixedmean)
   g_FastICA_mixedsig = g_FastICA_mixedsig';
 else
   g_FastICA_mixedsig = (g_FastICA_mixedsig + ...
			 g_FastICA_mixedmean * ...
			 ones(1,size(g_FastICA_mixedsig, 2)))';
   g_FastICA_mixedmean = [];
 end
 fastica_gui_cb("NewData");             // Data has been changed


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'DoPCA' then

 if ~isempty(g_FastICA_mixedsig)
   // We'll remove mean of the data here also just in case...
   if isempty(g_FastICA_mixedmean)
     [g_FastICA_mixedsig, g_FastICA_mixedmean] = remmean(g_FastICA_mixedsig);
   end
   
   // Do PCA interactively: ask the user for eigenvalues
   [g_FastICA_pca_E, g_FastICA_pca_D] = pcamat(g_FastICA_mixedsig, ...
					       0, 0, 'gui', ...
					       stripblanks(c_FastICA_verb_strV(g_FastICA_verbose,:)));
   
   newdim = size(g_FastICA_pca_D, 1);
   set(ht_FastICA_newDim, 'String', string(newdim));
   set(he_FastICA_numOfIC, 'String', string(newdim));
   g_FastICA_numOfIC = newdim;
   fastica_gui_cb("NullWhite");           // Whitening needs to be done again also
			       // but we'll do it when it's needed.
 else
   printf('Data not loaded yet!\n\n');
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'OrigDim' then

 fastica_gui_cb("NewData");

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'ShowMixed' then

 if ~isempty(g_FastICA_mixedsig)
   handle = findobj('Tag','f_FastICA_mix');  // Check if the window is already
   if isempty(handle)                        // open. If not then open it.
     figure('Tag', 'f_FastICA_mix', ...
	    'Figure_Name', 'FastICA: Plot data');
   else
     figure(handle);
     clf;		// clear the figure for next plots
   end
   if isempty(g_FastICA_mixedmean)
     icaplot('dispsig',g_FastICA_mixedsig, 0, 0, 0, 'Mixed signals');
   else
     icaplot('dispsig',g_FastICA_mixedsig + g_FastICA_mixedmean * ...
	     ones(1, size(g_FastICA_mixedsig, 2)), 0, 0, 0, 'Mixed signals');
   end
 else
   printf('Data not loaded yet!\n\n');
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'ShowWhite' then

 if ~isempty(g_FastICA_mixedsig)
   if isempty(g_FastICA_white_sig)     // if whitening is not done, we need to
     fastica_gui_cb("Whiten");                    // do it before we can display the
   end                                 // whitened signals
   
   handle = findobj('Tag','f_FastICA_white');  // Check if the window is already
   if isempty(handle)                          // open. If not then open it.
     figure('Tag', 'f_FastICA_white', ...
	    'Figure_Name', 'FastICA: Plot whitened');
   else
     figure(handle);
     clf;		// clear the figure for next plots
   end
   icaplot('dispsig',g_FastICA_white_sig,0,0,0,'Whitened signals');
 else
   printf('Data not loaded yet!\n\n');
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Whiten' then

 set(ht_FastICA_whiteStatus,'String','Computing...');
 
 // If PCA is not calculated, we'll have to calculate it now,
 // we'll do it without guestions - we don't reduce the dimension
 // here - but PCAMAT might reduce the dimension automatically.
 if isempty(g_FastICA_pca_E)
   // We'll remove mean of the data here also just in case...
   if isempty(g_FastICA_mixedmean)
     [g_FastICA_mixedsig, g_FastICA_mixedmean] = remmean(g_FastICA_mixedsig);
   end
   
   [g_FastICA_pca_E, g_FastICA_pca_D] = pcamat(g_FastICA_mixedsig, 1, ...
					       size(g_FastICA_mixedsig, ...
						    1), 'off', ...
					       stripblanks(c_FastICA_verb_strV(g_FastICA_verbose,:)));
   
   // Check if the dimension was reduced automatically
   newdim = size(g_FastICA_pca_D, 1);
   set(ht_FastICA_newDim, 'String', string(newdim));
   // Check if the numOfIC now has illegal value entered 
   // We do that by telling the program that there is new value 
   // entered for NumOfIC.
   fastica_gui_cb("ChangeNumOfIC");
 end
 
 // And now we can calculate whitening...
 [g_FastICA_white_sig, g_FastICA_white_wm, g_FastICA_white_dwm] = ...
								  whitenv(g_FastICA_mixedsig, g_FastICA_pca_E, g_FastICA_pca_D, stripblanks(c_FastICA_verb_strV(g_FastICA_verbose,:)));
 
 set (ht_FastICA_whiteStatus,'String','');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'ChangeApproach' then

 // Get the old value for g
 execstr('g_str = c_FastICA_g' +string(g_FastICA_approach)+ '_strV;');
 old_g = stripblanks(g_str(g_FastICA_g,:));
 
 // Get and set the new value for approach
 g_FastICA_approach = get(hpm_FastICA_approach, 'Value');
 
 // The possible values for g depend on the value of approach...
 execstr('g_str = c_FastICA_g' +string(g_FastICA_approach)+ '_strD;');
 set(hpm_FastICA_g, 'String', g_str);
 
 // Match the old g value from the new g values so that if the 
 // old_g can be found from the new values (anywhere), then set new g
 // to that value, and if it's not found then set the new value to 1.
 match = 0;
 execstr('g_str = c_FastICA_g'+ string(g_FastICA_approach)+ '_strV;');
 for i=1:size(g_str,1)
   if (old_g==stripblanks(g_str(i,:)))
     match = i;
   end
 end
 if match == 0
   match = 1;   // the old g is not availabe anymore, set g = 1.
 end
 g_FastICA_g = match;
 set(hpm_FastICA_g, 'Value', match);
 
 fastica_gui_cb("NullICA");      // The options are changed so we must calculate ICA again
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'ChangeNumOfIC' then

 // Get the new value... and store it later on after some checks
 numofic = eval(get(he_FastICA_numOfIC, 'String'));
 
 // The number of IC can't be less than 1 or more than the reduced dimension.
 numoficmax = eval(get(ht_FastICA_newDim, 'String'));
 if numofic < 1
   set(he_FastICA_numOfIC, 'String', '1');
   g_FastICA_numOfIC = 1;
 elseif numofic > numoficmax
   set(he_FastICA_numOfIC, 'String', string (numoficmax));
   g_FastICA_numOfIC = numoficmax;
 else
   g_FastICA_numOfIC = numofic;
 end
 
 fastica_gui_cb("NullICA");      // The options are changed so we must calculate ICA again
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'ChangeG' then

 // Get the new value for g.
 g_FastICA_g = get(hpm_FastICA_g, 'Value');
 
 fastica_gui_cb("NullICA");      // The options are changed so we must calculate ICA again
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'ChangeStab'

 // Get the new value for g.
 g_FastICA_stabilization = get(hpm_FastICA_stabilizat, 'Value');
 
 fastica_gui_cb("NullICA");      // The options are changed so we must calculate ICA again
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'AdvOpt' then

 handle = findobj('Tag','f_FastICAAdvOpt');
 if isempty(handle)                        // Check to see if the window is
   pos = get(hf_FastICA_MAIN, 'Position');     // already open...
   if ~isempty (pos),
     fastica_gui_adv(pos(1), pos(2));
   else
     fastica_gui_adv(0, 0);
   end
 else
   figure(handle)
 end
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'ShowICASig' then
 
 if ~isempty(g_FastICA_mixedsig)
   // If the IC's are not already calculated, we'll do it now
   if isempty(g_FastICA_ica_sig)
     fastica_gui_cb("DoFPICA");
   end
   
   // The signals may have been already displaued by the FPICA function
   // BUT the FPICA may also have shown either the basis of the filters
   // so the signals still need to be shown - besides the mean was added
   // in only later after FPICA
   
   // Also notice that in this version if there was something wrong in FPICA
   // Then the results are []. - We don't try to plot them!
   if ~isempty(g_FastICA_ica_sig')
     handle = findobj('Tag','f_FastICA_ica');  // Check if the window is already
     if isempty(handle)                        // open. If not then open it.
       figure('Tag', 'f_FastICA_ica', ...
	      'Figure_Name', 'FastICA: Plot ICs');
     else
       figure(handle);
       clf();		// clear the figure for next plots
     end
     
     icaplot('dispsig', g_FastICA_ica_sig, 0, ...
	     0, 0, 'Independent components');
   end
 else
   printf('Data not loaded yet!\n\n');
 end
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'DoFPICA' then

 fastica_gui_cb("DisableButtons");
 g_FastICA_interrupt = 0;
 
 if ~isempty(g_FastICA_mixedsig)
   if isempty(g_FastICA_white_sig)     // We need the whitened signal here
     fastica_gui_cb("Whiten");
   end
   
   set(ht_FastICA_icaStatus,'String','Computing...');
   
   // The possible values for g depend on approach
   execstr('g_str = c_FastICA_g'+ string(g_FastICA_approach) +'_strV;');
   
   // We'll contruct a command string which we'll later on evaluate...
   // This is where the Fixed point algorithm is used.
   command_str = ['[g_FastICA_ica_A,g_FastICA_ica_W]=' ...
		  'fpica(g_FastICA_white_sig,' ... 
		  'g_FastICA_white_wm,' ... 
		  'g_FastICA_white_dwm,' ...
		  '''' stripblanks(c_FastICA_appr_strV(g_FastICA_approach,:)) ...
		  ''',' ...
		  'g_FastICA_numOfIC,' ...
		  '''' stripblanks(g_str(g_FastICA_g,:)) ''',' ...
		  '''' ...
		  stripblanks(c_FastICA_finetune_strV(g_FastICA_finetune,:)) ...
		  ''',' ...
		  'g_FastICA_a1,' ...
		  'g_FastICA_a2,' ...
		  'g_FastICA_myy,' ...
		  '''' ...
		  stripblanks(c_FastICA_stabili_strV(g_FastICA_stabilization,:)) ...
		  ''',' ...
		  'g_FastICA_epsilon,' ...
		  'g_FastICA_maxNumIte,' ...
		  'g_FastICA_maxFinetune,' ...
		  '''' stripblanks(c_FastICA_iSta_strV(g_FastICA_initState,:)) ...
		  ''',' ...
		  'g_FastICA_initGuess,' ...
		  'g_FastICA_sampleSize,' ...
		  '''' stripblanks(c_FastICA_dMod_strV(g_FastICA_displayMo,:)) ...
		  ''',' ...
		  'g_FastICA_displayIn,' ...
		  '''' stripblanks(c_FastICA_verb_strV(g_FastICA_verbose,:)) ...
		  ''');'];
   

   // If the user wants to plot while computing...
   // let's at least plot it to the right figure then
   if ~(stripblanks(c_FastICA_dMod_strV(g_FastICA_displayMo,:))=='off')
     handle = findobj('Tag','f_FastICA_ica');  // Check if the window is already
     if isempty(handle)                        // open. If not then open it.
       figure('Tag', 'f_FastICA_ica', ...
	      'Figure_Name', 'FastICA: Plot ICs');
     else
       figure(handle);
       clf;		// clear the figure for next plots
     end
   end
   
   // ... and so let's do it...
   execstr(strcat(command_str));
   
   // Also notice that in this version if there was something wrong in FPICA
   // Then the results are [].
   if ~isempty(g_FastICA_ica_W)
     // Add the mean back in.
     g_FastICA_ica_sig = g_FastICA_ica_W * g_FastICA_mixedsig ...
			 + (g_FastICA_ica_W * g_FastICA_mixedmean) ...
			 * ones(1,size(g_FastICA_mixedsig, 2));
     set (ht_FastICA_icaStatus,'String','Done');
   else
     fastica_gui_cb("NullICA");  // set icasig=[] and do what ever needs to be done then
   end
   
   if ~(g_FastICA_interrupt)
     fastica_gui_cb("EnableButtons");
   end
 else
   printf('Data not loaded yet!\n\n');
 end


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Interrupt' then
 g_FastICA_interrupt = 1;
 set(ht_FastICA_icaStatus,'String','Interrupted');
 fastica_gui_cb("EnableButtons");

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'DisableButtons' then
 set(findobj('Tag','b_Transpose'),'Enable','off');
 set(findobj('Tag','b_ShowMixed'),'Enable','off');
 set(findobj('Tag','b_DoPCA'),'Enable','off');
 set(findobj('Tag','b_OrigDim'),'Enable','off');
 set(findobj('Tag','b_ShowWhite'),'Enable','off');
 set(findobj('Tag','b_advOpt'),'Enable','off');
 set(findobj('Tag','b_ShowICASig'),'Enable','off');
 set(findobj('Tag','b_LoadData'),'Enable','off');
 set(findobj('Tag','b_DoFPICA'),'Enable','off');
 set(findobj('Tag','b_SaveData'),'Enable','off');
 set(findobj('Tag','b_Quit'),'Enable','off');
 set(findobj('Tag','b_Interrupt'),'Visible','on');
 drawnow;


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'EnableButtons' then
 set(findobj('Tag','b_Transpose'),'Enable','on');
 set(findobj('Tag','b_ShowMixed'),'Enable','on');
 set(findobj('Tag','b_DoPCA'),'Enable','on');
 set(findobj('Tag','b_OrigDim'),'Enable','on');
 set(findobj('Tag','b_ShowWhite'),'Enable','on');
 set(findobj('Tag','b_advOpt'),'Enable','on');
 set(findobj('Tag','b_ShowICASig'),'Enable','on');
 set(findobj('Tag','b_LoadData'),'Enable','on');
 set(findobj('Tag','b_DoFPICA'),'Enable','on');
 set(findobj('Tag','b_SaveData'),'Enable','on');
 set(findobj('Tag','b_Quit'),'Enable','on');
 set(findobj('Tag','b_Interrupt'),'Visible','off');
 drawnow;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'SaveData' then

 handle = findobj('Tag','f_FastICASave');  // Check if the window is already
 if isempty(handle)                        // open. If not then open it.
   pos = get(hf_FastICA_MAIN, 'Position');
   if ~isempty (pos),
     fastica_gui_s(pos(1), pos(2));
   else
     fastica_gui_s(0, 0);
   end
 else
   figure(handle);                       // window. If it wasn't then
 end
 
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Quit' then

 // We'll close the other dialogs if they are open.
 Tags = ['f_FastICALoad  '
	 'f_FastICAAdvOpt'
	 'f_FastICASave  '
	 'f_FastICA_mix  '
	 'f_FastICA_white'
	 'f_FastICA_ica  '];
 for i=1:size(Tags,1)
   handle = findobj('Tag', stripblanks(Tags(i,:)));
   if ~isempty(handle)
     close(handle);
   end
 end

 // Close this window
 close(hf_FastICA_MAIN);
 hf_FastICA_MAIN=[];
 // Clear the used global variables.
 fastica_gui_cg();

 // Use return to avoid reaching the watchoff statement at the end
 // (There used to be a 'break' statement here, but it resulted in
 // errors in more recent version of Matlab -- jarmo)
 return;
 // ... and we're done.

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'About' then

 fastica_gui_help('gui_cb_about');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Help' then

 fastica_gui_help('gui_cb_help');
  
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

end    // switch

//watchoff (watchonInFigure);
endfunction


function fastica_gui_cg()
//
// This function is needed by FASTICAG

// This file just removes the clearglobal variables
// that are used in FASTICAG from the memory

// @(#)$Id: gui_cg.m,v 1.2 2003/04/05 14:23:57 jarmo Exp $

  clearglobal c_FastICA_appr_strD;
  clearglobal c_FastICA_appr_strV;
  clearglobal c_FastICA_dMod_strD;
  clearglobal c_FastICA_dMod_strV;
  clearglobal c_FastICA_finetune_strD;
  clearglobal c_FastICA_finetune_strV;
  clearglobal c_FastICA_g1_strD;
  clearglobal c_FastICA_g1_strV;
  clearglobal c_FastICA_g2_strD;
  clearglobal c_FastICA_g2_strV;
  clearglobal c_FastICA_iSta_strD;
 clearglobal c_FastICA_iSta_strV;
  clearglobal c_FastICA_stabili_strD;
  clearglobal c_FastICA_stabili_strV;
  clearglobal c_FastICA_verb_strD;
  clearglobal c_FastICA_verb_strV;
  clearglobal g_FastICA_a1;
  clearglobal g_FastICA_a2;
  clearglobal g_FastICA_approach;
  clearglobal g_FastICA_displayIn;
  clearglobal g_FastICA_displayMo;
  clearglobal g_FastICA_epsilon;
  clearglobal g_FastICA_finetune;
  clearglobal g_FastICA_g;
  clearglobal g_FastICA_ica_A;
  clearglobal g_FastICA_ica_W;
  clearglobal g_FastICA_ica_sig;
  clearglobal g_FastICA_initGuess;
  clearglobal g_FastICA_initState;
  clearglobal g_FastICA_interrupt;
  clearglobal g_FastICA_loadType;
  clearglobal g_FastICA_maxFinetune;
  clearglobal g_FastICA_maxNumIte;
  clearglobal g_FastICA_mixedmean;
  clearglobal g_FastICA_mixedsig;
  clearglobal g_FastICA_myy;
  clearglobal g_FastICA_numOfIC;
  clearglobal g_FastICA_pca_D;
  clearglobal g_FastICA_pca_E;
  clearglobal g_FastICA_sampleSize;
  clearglobal g_FastICA_stabilization;
  clearglobal g_FastICA_verbose;
  clearglobal g_FastICA_white_dwm;
  clearglobal g_FastICA_white_sig;
  clearglobal g_FastICA_white_wm;
  clearglobal hb_FastICA_initGuess;
  clearglobal he_FastICA_a1;
  clearglobal he_FastICA_a2;
  clearglobal he_FastICA_displayInterv;
  clearglobal he_FastICA_epsilon;
  clearglobal he_FastICA_file;
  clearglobal he_FastICA_suffix;
  clearglobal he_FastICA_maxFinetune;
  clearglobal he_FastICA_maxIterations;
  clearglobal he_FastICA_myy;
  clearglobal he_FastICA_numOfIC;
  clearglobal he_FastICA_sampleSize;
  clearglobal hf_FastICA_MAIN;
  clearglobal hf_FastICA_AdvOpt;
  clearglobal hf_FastICA_Load;
  clearglobal hf_FastICA_Save;
  clearglobal hpm_FastICA_approach;
  clearglobal hpm_FastICA_displayMode;
  clearglobal hpm_FastICA_finetune;
  clearglobal hpm_FastICA_g;
  clearglobal hpm_FastICA_initState;
  clearglobal hpm_FastICA_stabilizat;
  clearglobal hpm_FastICA_verbose;
  clearglobal ht_FastICA_dim;
  clearglobal ht_FastICA_icaStatus;
  clearglobal ht_FastICA_initGuess;
  clearglobal ht_FastICA_mixedStatus;
  clearglobal ht_FastICA_newDim;
  clearglobal ht_FastICA_numOfSamp;
  clearglobal ht_FastICA_whiteStatus;

  clearglobal hf_FastICA_MAIN;

endfunction







function fastica_gui_adv(x, y)
//
// This file is needed by FASTICAG

// This is the advanced options -dialog

// @(#)$Id: gui_adv.m,v 1.4 2004/07/27 13:09:26 jarmo Exp $

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables

// Handle to the window
global hf_FastICA_AdvOpt;

// Handles to some of the controls in window
global hpm_FastICA_finetune;
global he_FastICA_a1;
global he_FastICA_a2;
global he_FastICA_myy;
global he_FastICA_epsilon;
global he_FastICA_maxIterations;
global he_FastICA_maxFinetune;
global he_FastICA_sampleSize
global hpm_FastICA_initState;
global hb_FastICA_initGuess;
global ht_FastICA_initGuess;
global hpm_FastICA_displayMode;
global he_FastICA_displayInterv;
global hpm_FastICA_verbose;

// Some of the main variables needed
global g_FastICA_initGuess;
global g_FastICA_finetune;
global g_FastICA_a1;
global g_FastICA_a2;
global g_FastICA_myy;
global g_FastICA_epsilon;
global g_FastICA_maxNumIte;
global g_FastICA_maxFinetune;
global g_FastICA_initState;
global g_FastICA_sampleSize;
global g_FastICA_displayMo;
global g_FastICA_displayIn;
global g_FastICA_verbose;

global c_FastICA_appr_strD;
global c_FastICA_appr_strV;
global c_FastICA_finetune_strD;
global c_FastICA_finetune_strV;
global c_FastICA_iSta_strD;
global c_FastICA_iSta_strV;
global c_FastICA_dMod_strD;
global c_FastICA_dMod_strV;
global c_FastICA_verb_strD;
global c_FastICA_verb_strV;


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Configuration options
FIGURENAME = 'FastICA: advanced options';
FIGURETAG = 'f_FastICAAdvOpt';
FIGURESIZE = [x y 450 280];

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Check to see if this figure is already open - it should not!
// Can't have more than one copy - otherwise the global
// variables and handles can get mixed up.
if ~isempty(findobj('Tag',FIGURETAG))
  error('Error: advanced options dialog already open!');
end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initialize some of the controls' values

// Did we already load some initial guess
pm_initState_Value = g_FastICA_initState;
if isempty(g_FastICA_initGuess) | (g_FastICA_initGuess == 1)
  t_initGuess_String = 'Not loaded';
else
  t_initGuess_String = 'Loaded';
end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create the figure
a = figure('Figure_Name', FIGURENAME, ...
	   'Tag', FIGURETAG, ...
	   'Position', FIGURESIZE);
//set (a, 'Resize', 'off');

hf_FastICA_AdvOpt = a;

//set(hf_FastICA_AdvOpt, 'HandleVisibility', 'callback');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// From here on it get's ugly as I have not had time to clean it up


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create the frames
pos_l=2;
pos_w=FIGURESIZE(3)-4;
pos_h=FIGURESIZE(4)-4;
pos_t=2;
h_f_adv_background = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_adv_background');

pos_w=120;
pos_l=FIGURESIZE(3)-(pos_w+2+2);
pos_h=FIGURESIZE(4)-2*4;
pos_t=4;
h_f_adv_side = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_adv_side');

pos_l=4;
pos_w=FIGURESIZE(3)-8-pos_w-2;
pos_h=FIGURESIZE(4)-8;
pos_t=4;
h_f_advopt = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_advopt');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_advopt
bgc = get(h_f_advopt, 'BackgroundColor');

pos_w1=230;
pos_w2=70;

pos_frame=get(h_f_advopt, 'Position');
pos_h = 20;
pos_t = pos_frame(2) + pos_frame(4) - pos_h - 6;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Fine-tune (g)', ...
  'Style','text', ...
  'Tag','t_727');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
hpm_FastICA_finetune = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',c_FastICA_finetune_strD, ...
  'Style','popupmenu', ...
  'Tag','pm_finetune', ...
  'Value',g_FastICA_finetune);

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Parameter a1 (g = ''tanh'')', ...
  'Style','text', ...
  'Tag','t_22');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
he_FastICA_a1 = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','right', ...
  'Callback','gui_advc Checka1', ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',string(g_FastICA_a1), ...
  'Style','edit', ...
  'Tag','e_a1');

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Parameter a2 (g = ''gauss'')', ...
  'Style','text', ...
  'Tag','t_222');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
he_FastICA_a2 = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','right', ...
  'Callback','gui_advc Checka2', ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',string(g_FastICA_a2), ...
  'Style','edit', ...
  'Tag','e_a2');

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','mu (step size)', ...
  'Style','text', ...
  'Tag','t_223');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
he_FastICA_myy = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','right', ...
  'Callback','gui_advc CheckMyy', ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',string(g_FastICA_myy), ...
  'Style','edit', ...
  'Tag','e_myy');

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','epsilon (stopping criterion)', ...
  'Style','text', ...
  'Tag','t_23');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
he_FastICA_epsilon = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','right', ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',string(g_FastICA_epsilon), ...
  'Style','edit', ...
  'Tag','e_epsilon');

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Maximum number of iterations', ...
  'Style','text', ...
  'Tag','t_24');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
he_FastICA_maxIterations = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','right', ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',string(g_FastICA_maxNumIte), ...
  'Style','edit', ...
  'Tag','e_maxIterations');

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Maximum iterations in fine-tuning', ...
  'Style','text', ...
  'Tag','t_2412');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
he_FastICA_maxFinetune = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','right', ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',string(g_FastICA_maxFinetune), ...
  'Style','edit', ...
  'Tag','e_maxFinetune');

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Sample size (proportion)', ...
  'Style','text', ...
  'Tag','t_224');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
he_FastICA_sampleSize = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','right', ...
  'Callback','gui_advc CheckSampleSize', ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',string(g_FastICA_sampleSize), ...
  'Style','edit', ...
  'Tag','e_sampleSize');

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Initial state', ...
  'Style','text', ...
  'Tag','t_25');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
hpm_FastICA_initState = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',c_FastICA_iSta_strD, ...
  'Style','popupmenu', ...
  'Tag','pm_initState', ...
  'Value',pm_initState_Value);

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
hb_FastICA_initGuess = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','gui_advc loadGuess', ...
  'Position',[pos_l pos_t (pos_w1-60) pos_h], ...
  'String','Load initial guess', ...
  'UserData', g_FastICA_initGuess, ...
  'Tag','b_LoadGuess');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
ht_FastICA_initGuess = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[(pos_l) pos_t (pos_w2) pos_h], ...
  'String',t_initGuess_String, ...
  'Style','text', ...
  'Tag','t_initGuess');

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Display mode', ...
  'Style','text', ...
  'Tag','t_27');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
hpm_FastICA_displayMode = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',c_FastICA_dMod_strD, ...
  'Style','popupmenu', ...
  'Tag','pm_displayMode', ...
  'Value',g_FastICA_displayMo);

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Iterations between displays', ...
  'Style','text', ...
  'Tag','t_28');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
he_FastICA_displayInterv = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','right', ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',string(g_FastICA_displayIn), ...
  'Style','edit', ...
  'Tag','e_displayInterval');

pos_t = pos_t - pos_h;
pos_l = pos_frame(1) + 6;
b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w1 pos_h], ...
  'String','Verbose', ...
  'Style','text', ...
  'Tag','t_29');

pos_l = pos_frame(1) + pos_frame(3) - 6 - pos_w2;
hpm_FastICA_verbose = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'Position',[pos_l pos_t pos_w2 pos_h], ...
  'String',c_FastICA_verb_strD, ...
  'Style','popupmenu', ...
  'Tag','pm_verbose', ...
  'Value',g_FastICA_verbose);

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_adv_side
pos_vspace = 6;
pos_hspace = 10;
pos_frame = get(h_f_adv_side, 'Position');
pos_w = 100;
pos_h = 30;
pos_l = pos_frame(1) + pos_hspace;
pos_t = pos_frame(2) + pos_frame(4) - pos_h - pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','gui_advc OK', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','OK', ...
  'Tag','b_advOK');

pos_t=pos_t-pos_h-pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','gui_advc Cancel', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Cancel', ...
  'Tag','b_advCancel');

pos_t=pos_t-pos_h-pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','gui_advc Default', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Default', ...
  'Tag','b_advDefault');

pos_t=pos_t-pos_h-pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','gui_advc Apply', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Apply', ...
  'Tag','b_advApply');

pos_t = pos_frame(2) + pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','gui_advc Help', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Help', ...
  'Tag','b_advHelp');

endfunction















function fastica_gui_advc (action)
//
// This file is needed by FASTICAG

// This file holds the callbacks for advanced options -dialog

// @(#)$Id: gui_advc.m,v 1.3 2003/09/08 11:28:58 jarmo Exp $

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables

// Handle to the window
global hf_FastICA_AdvOpt;

// Handles to some of the controls in window
global hpm_FastICA_finetune;
global he_FastICA_a1;
global he_FastICA_a2;
global he_FastICA_myy;
global he_FastICA_epsilon;
global he_FastICA_maxIterations;
global he_FastICA_maxFinetune;
global he_FastICA_sampleSize;
global hpm_FastICA_initState;
global hb_FastICA_initGuess;
global ht_FastICA_initGuess;
global hpm_FastICA_displayMode;
global he_FastICA_displayInterv;
global hpm_FastICA_verbose;

// Needed handles to the main window
global hf_FastICA_MAIN;
global ht_FastICA_icaStatus;
global ht_FastICA_numOfSamp;
global hpm_FastICA_stabilizat;

// Some of the main variables needed
global g_FastICA_initGuess;
global g_FastICA_numOfIC;
global g_FastICA_finetune;
global g_FastICA_a1;
global g_FastICA_a2;
global g_FastICA_myy;
global g_FastICA_epsilon;
global g_FastICA_maxNumIte;
global g_FastICA_maxFinetune;
global g_FastICA_initState;
global g_FastICA_sampleSize;
global g_FastICA_displayMo;
global g_FastICA_displayIn;
global g_FastICA_verbose;

global c_FastICA_iSta_strV;

// What is the load type of load dialog
global g_FastICA_loadType;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This should not take long...
//watchonInFigure = watchon;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
select action
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Checka1' then

 e_a1_val = eval(get(he_FastICA_a1, 'String'));
 if e_a1_val <= 0
   set(he_FastICA_a1, 'String', '0.1');
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Checka2' then
  
 e_a2_val = eval(get(he_FastICA_a2, 'String'));
 if e_a2_val <= 0
   set(he_FastICA_a2, 'String', '0.1');
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'CheckMyy' then

 e_myy_val = eval(get(he_FastICA_myy, 'String'));
 if e_myy_val <= 0
   set(he_FastICA_myy, 'String', '0.1');
 elseif e_myy_val > 1
   set(he_FastICA_myy, 'String', '1');
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'CheckSampleSize' then
  
 e_sampleSize_val = eval(get(he_FastICA_sampleSize, 'String'));
 if e_sampleSize_val > 1
   set(he_FastICA_sampleSize, 'String', '1');
 else 
   numOfSamp = eval(get(ht_FastICA_numOfSamp, 'String'));
   if numOfSamp < 1000
     set(he_FastICA_sampleSize, 'String', '1');
     printf('Can''t reduce sample size. Already less than 1000 samples.\n');
   elseif (e_sampleSize_val * numOfSamp) < 1000
     printf('Can''t reduce sample size to less than 1000 samples.\n');
     set(he_FastICA_sampleSize, 'String', sprintf('%0.3f',1000/numOfSamp));
   end
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'loadGuess' then
 
 handle = findobj('Tag','f_FastICALoad');  // Check if the window is already
 if isempty(handle)                        // open. If not then open it.
   pos = get(hf_FastICA_MAIN, 'Position');
   fastica_gui_l(pos(1), pos(2),'LoadGuess');
 else
   if strcmp(g_FastICA_loadType, 'guess')  // Check if it was the same load
     figure(handle);                       // window. If it wasn't then
   else                                    // close the other window first
     close(handle);                        // and then open the load window
     printf('''Load data'' -dialog closed!\n');
     pos = get(hf_FastICA_MAIN, 'Position');
     fastica_gui_l(pos(1), pos(2),'LoadGuess');
   end
 end
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'OK' then
  
 fastica_gui_advc Apply;

 close(hf_FastICA_AdvOpt);

 // Use return to avoid reaching the watchoff statement at the end
 // (There used to be a 'break' statement here, but it resulted in
 // errors in more recent version of Matlab -- jarmo)
 return;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Apply' then

 newValues = 0;

 val = g_FastICA_finetune;
 g_FastICA_finetune = get(hpm_FastICA_finetune, 'Value');
 if (g_FastICA_finetune ~= val)
   newValues = 1;
 end
 
 val = g_FastICA_a1;
 g_FastICA_a1 = eval(get(he_FastICA_a1, 'String'));
 if (g_FastICA_a1 ~= val)
   newValues = 1;
 end

 val = g_FastICA_a2;
 g_FastICA_a2 = eval(get(he_FastICA_a2, 'String'));
 if (g_FastICA_a2 ~= val)
   newValues = 1;
 end

 val = g_FastICA_myy;
 g_FastICA_myy = eval(get(he_FastICA_myy, 'String'));
 if (g_FastICA_myy ~= val)
   newValues = 1;
 end

 // If myy < 1 then will use stabilazed code, and we don't care 
 // about the parameter stabilization :-)
 if (g_FastICA_myy == 1)
   set(hpm_FastICA_stabilizat, 'Enable', 'on');
 else
   set(hpm_FastICA_stabilizat, 'Enable', 'off');
 end

 val = g_FastICA_epsilon;
 g_FastICA_epsilon = eval(get(he_FastICA_epsilon, 'String'));
 if (g_FastICA_epsilon ~= val)
   newValues = 1;
 end

 val = g_FastICA_maxNumIte;
 g_FastICA_maxNumIte = eval(get(he_FastICA_maxIterations, 'String'));
 if (g_FastICA_maxNumIte ~= val)
   newValues = 1;
 end
 
 val = g_FastICA_maxFinetune;
 g_FastICA_maxFinetune = eval(get(he_FastICA_maxFinetune, 'String'));
 if (g_FastICA_maxFinetune ~= val)
   newValues = 1;
 end
 
 val = g_FastICA_sampleSize;
 g_FastICA_sampleSize = eval(get(he_FastICA_sampleSize, 'String'));
 if (g_FastICA_sampleSize ~= val)
   newValues = 1;
 end

 val = g_FastICA_initState;
 g_FastICA_initState = get(hpm_FastICA_initState, 'Value');
 if (g_FastICA_initState ~= val)
   newValues = 1;
 end

 val = g_FastICA_initGuess;
 g_FastICA_initGuess = get(hb_FastICA_initGuess, 'UserData');
 if min(bool2s(size(val) == size(g_FastICA_initGuess))) == 0
   newValues = 1;
 else
   if (g_FastICA_initGuess ~= val)
     newValues = 1;
   end
 end
 
 g_FastICA_displayMo = get(hpm_FastICA_displayMode, 'Value');
 g_FastICA_displayIn = eval(get(he_FastICA_displayInterv, 'String'));
 g_FastICA_verbose = get(hpm_FastICA_verbose, 'Value');
 
 if newValues == 1
   fastica_gui_cb NullICA;
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Cancel' then
  
 close(hf_FastICA_AdvOpt);

 // Use return to avoid reaching the watchoff statement at the end
 // (There used to be a 'break' statement here, but it resulted in
 // errors in more recent version of Matlab -- jarmo)
 return;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Default' then
  
 // set default values to controls
 set(hpm_FastICA_finetune, 'Value',5);
 set(he_FastICA_a1, 'String','1');
 set(he_FastICA_a2, 'String','1');
 set(he_FastICA_myy, 'String','1');
 set(he_FastICA_epsilon, 'String','0.0001');
 set(he_FastICA_maxIterations, 'String','1000');
 set(he_FastICA_sampleSize, 'String','1');
 set(hpm_FastICA_initState, 'Value',1);
 set(hpm_FastICA_displayMode, 'Value',1);
 set(he_FastICA_displayInterv, 'String','1');
 set(hpm_FastICA_verbose, 'Value',1);

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Help' then

 fastica_gui_help('gui_advc');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
end    // switch

//watchoff (watchonInFigure);
endfunction







function fastica_gui_l (x, y,dataMode)
//
// This file is needed by FASTICAG

// The load dialog for loading new data
// and new initial guess.

// @(#)$Id: gui_l.m,v 1.4 2004/07/27 13:09:26 jarmo Exp $

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables

// Handle to the window
global hf_FastICA_Load;

// Handles to some of the controls in window
global he_FastICA_file;

// What is the load type of load dialog
global g_FastICA_loadType;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Configuration options
FIGURENAME = 'FastICA: Load';
FIGURETAG = 'f_FastICALoad';
FIGURESIZE = [x y 450 150];

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Check to see if this figure is already open - it should not!
// Can't have more than one copy - otherwise the global
// variables and handles can get mixed up.
if ~isempty(findobj('Tag',FIGURETAG))
  error('Error: load dialog already open!');
end


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initialize some of the controls' values

// What are we loading - who is calling?
//caller = get(gcf, 'CurrentObject');

select dataMode
 case 'LoadData'     then                        // Do we load new data...
  loadString = 'Load data from variable in Matlab.';
  g_FastICA_loadType = 'data';
  FIGURENAME = 'FastICA: Load data';

 case 'LoadGuess'    then                        // ... or new initial guess?
  loadString = 'Load initial guess for mixing matrix A from variable in Matlab.';
  g_FastICA_loadType = 'guess';
  FIGURENAME = 'FastICA: Load initial guess';
end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create the figure
a = figure('Figure_Name', FIGURENAME, ...
	   'Tag', FIGURETAG, ...
	   'Position', FIGURESIZE);
//set (a, 'Resize', 'off');

hf_FastICA_Load = a;

//set(hf_FastICA_Load, 'HandleVisibility', 'callback');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// From here on it get's ugly as I have not had time to clean it up


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create the frames
pos_l=2;
pos_w=FIGURESIZE(3)-4;
pos_h=FIGURESIZE(4)-4;
pos_t=2;
h_f_load_background = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_load_background');

pos_w=120;
pos_l=FIGURESIZE(3)-(pos_w+2+2);
pos_h=FIGURESIZE(4)-2*4;
pos_t=4;
h_f_load_side = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_load_side');

pos_l=4;
pos_w=FIGURESIZE(3)-8-pos_w-2;
pos_h=FIGURESIZE(4)-8;
pos_t=4;
h_f_load = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_load');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_load
bgc = get(h_f_load, 'BackgroundColor');

pos_w=230;

pos_frame=get(h_f_load, 'Position');
pos_h = 40;
pos_t = pos_frame(2) + pos_frame(4) - pos_h - 6;
pos_l = pos_frame(1) + 6;

b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String',loadString, ...
  'Style','text', ...
  'Tag','t_93');

pos_h = 20;
pos_t = pos_t - pos_h - 10;
pos_l = pos_frame(1) + 6;

b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Name of the variable:', ...
  'Style','text', ...
  'Tag','t_92');

pos_w = 200;
pos_l = pos_l + 30;
pos_t = pos_t - pos_h;
he_FastICA_file = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','', ...
  'Style','edit', ...
  'Tag','e_file');


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_load_side
pos_vspace = 6;
pos_hspace = 10;
pos_frame = get(h_f_load_side, 'Position');
pos_w = 100;
pos_h = 30;
pos_l = pos_frame(1) + pos_hspace;
pos_t = pos_frame(2) + pos_frame(4) - pos_h - pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_lc Load', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Load', ...
  'Tag','b_lLoad');

pos_t=pos_t-pos_h-pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_lc Cancel', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Cancel', ...
  'Tag','b_lCancel');

pos_t = pos_frame(2) + pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_lc Help', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Help', ...
  'Tag','b_lHelp');
endfunction






function fastica_gui_lc (action)
//
// This file is used by FASTICAG

// This file holds the callbacks for load-dialog

// @(#)$Id: gui_lc.m,v 1.4 2003/09/11 12:01:19 jarmo Exp $

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables

// Handle to the window
global hf_FastICA_Load;

// Handles to some of the controls in window
global he_FastICA_file;

// Needed handles from the main figure
global ht_FastICA_mixedStatus;

// Needed handles from the advOpt figure
global hb_FastICA_initGuess;
global ht_FastICA_initGuess;
global hpm_FastICA_initState;

// The needed main variables
global g_FastICA_mixedsig;
global g_FastICA_mixedmean;

// What is the load type of load dialog
global g_FastICA_loadType;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This should not take long...
//watchonInFigure = watchon;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
select action
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Load' then
 
 varName = get(he_FastICA_file, 'String');      // The name of the variable to be loaded
 //command=['evalin(''base'',''assignin(''''caller'''',''''data'''',' varName ')'')'];

 //eval(command,'printf(''Variable not found in MATLAB workspace, data not loaded!\n'');data=[];');                          // Variable is copyed to 'data'
 execstr("data="+varName);
 if length (size (data)) > 2,
   printf (['Input data can not have more than two dimensions, data' ...
	     ' not loaded.\n']);
   data = [];
 end

 if or (or (isnan (data))),
   printf ('Input data contains NaN''s, data not loaded.\n');
   data = [];
 end


 if isempty(data)  // if there was no name given...
   //watchoff (watchonInFigure);
   // There used to be a 'break' statement here, but it resulted in
   // errors in more recent version of Matlab -- jarmo
   return;
 end
 select g_FastICA_loadType
  case  'data'    then                            // New data
   g_FastICA_mixedsig = data;
   if ~(type (g_FastICA_mixedsig)==1)
     printf ('Warning: converting input data into regular (double) precision.\n');
     g_FastICA_mixedsig = double (g_FastICA_mixedsig);
   end

   set(ht_FastICA_mixedStatus, 'String', '');
   g_FastICA_mixedmean = [];                 // New data - so that means ...
   fastica_gui_cb NewData;                             
   
  case 'guess'   then                             // New initial guess
   set(hb_FastICA_initGuess, 'UserData', data);     // Since we loaded new initial
   set(ht_FastICA_initGuess, 'String', 'Loaded');   // guess, we wan't to use it too
   set(hpm_FastICA_initState, 'Value', 2);          // ... set initState to 'guess'
 end

 close(hf_FastICA_Load);                       // close the dialog

 // Use return to avoid reaching the watchoff statement at the end
 // (There used to be a 'break' statement here, but it resulted in
 // errors in more recent version of Matlab -- jarmo)
 return;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Cancel' then
 
 close(hf_FastICA_Load);                       // do nothing just exit

 // Use return to avoid reaching the watchoff statement at the end
 // (There used to be a 'break' statement here, but it resulted in
 // errors in more recent version of Matlab -- jarmo)
 return;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Help' then

 // Which help do we show?
 select g_FastICA_loadType
  case 'data' then
   fastica_gui_help('gui_lc_data');
  case 'guess' then
   fastica_gui_help('gui_lc_guess');
 end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
end    // switch

//watchoff (watchonInFigure);
endfunction









function fastica_gui_s (x, y)
//
// This file is used by FASTICAG

// The save dialog for saving the results

// @(#)$Id: gui_s.m,v 1.4 2004/07/27 13:09:26 jarmo Exp $

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables

// Handle to the window
global hf_FastICA_Save;

// Handles to some of the controls in window
global he_FastICA_suffix;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Configuration options
FIGURENAME = 'FastICA: Save results';
FIGURETAG = 'f_FastICASave';
FIGURESIZE = [x y 450 150];

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Check to see if this figure is already open - it should not!
// Can't have more than one copy - otherwise the global
// variables and handles can get mixed up.
if ~isempty(findobj('Tag',FIGURETAG))
  error('Error: load dialog already open!');
end


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initialize some of the controls' values

saveString = 'Save results as variables in Scilab workspace as global structure.';
promptString = 'Suffix to identify the results:';
helpString = 'If you give e.g. ''FastICA'', the variables will be called FastICA.A, FastICA.W, etc.';

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create the figure
a = figure('Figure_Name', FIGURENAME, ...
	   'Tag', FIGURETAG, ...
	   'Position', FIGURESIZE);
//set (a, 'Resize', 'off');

hf_FastICA_Save = a;

//set(hf_FastICA_Save, 'HandleVisibility', 'callback');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// From here on it get's ugly as I have not had time to clean it up


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create the frames
pos_l=2;
pos_w=FIGURESIZE(3)-4;
pos_h=FIGURESIZE(4)-4;
pos_t=2;
h_f_save_background = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_save_background');

pos_w=120;
pos_l=FIGURESIZE(3)-(pos_w+2+2);
pos_h=FIGURESIZE(4)-2*4;
pos_t=4;
h_f_save_side = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_save_side');

pos_l=4;
pos_w=FIGURESIZE(3)-8-pos_w-2;
pos_h=FIGURESIZE(4)-8;
pos_t=4;
h_f_save = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'Style','frame', ...
  'Tag','f_save');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_save
bgc = get(h_f_save, 'BackgroundColor');

pos_w=230;

pos_frame=get(h_f_save, 'Position');
pos_h = 40;
pos_t = pos_frame(2) + pos_frame(4) - pos_h - 6;
pos_l = pos_frame(1) + 6;

b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String',saveString, ...
  'Style','text', ...
  'Tag','t_93');

pos_h = 20;
pos_t = pos_t - pos_h - 10;
pos_l = pos_frame(1) + 6;

b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String',promptString, ...
  'Style','text', ...
  'Tag','t_92');

pos_w = 200;
pos_l = pos_l + 30;
pos_t = pos_t - pos_h;
he_FastICA_suffix = uicontrol('Parent',a, ...
  'BackgroundColor',[1 1 1], ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','', ...
  'Style','edit', ...
  'Tag','e_suffix');

pos_w = pos_frame(3) - 12;
pos_h = 30;
pos_t = pos_frame(2) + 6;
pos_l = pos_frame(1) + 6;

b = uicontrol('Parent',a, ...
  'BackgroundColor',bgc, ...
  'HorizontalAlignment','left', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String',helpString, ...
  'Style','text', ...
  'Tag','t_97');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Controls in f_save_side
pos_vspace = 6;
pos_hspace = 10;
pos_frame = get(h_f_save_side, 'Position');
pos_w = 100;
pos_h = 30;
pos_l = pos_frame(1) + pos_hspace;
pos_t = pos_frame(2) + pos_frame(4) - pos_h - pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_sc Save', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Save', ...
  'Tag','b_sSave');

pos_t=pos_t-pos_h-pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_sc Cancel', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Cancel', ...
  'Tag','b_sCancel');

pos_t = pos_frame(2) + pos_vspace;
b = uicontrol('Parent',a, ...
  'BackgroundColor',[0.701961 0.701961 0.701961], ...
  'Callback','fastica_gui_sc Help', ...
  'Position',[pos_l pos_t pos_w pos_h], ...
  'String','Help', ...
  'Tag','b_sHelp');

endfunction







function fastica_gui_sc (action)
//
// This file is used by FASTICAG

// This file holds the callbacks for save-dialog

// @(#)$Id: gui_sc.m,v 1.3 2003/09/08 11:28:59 jarmo Exp $

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables

// Handle to the window
global hf_FastICA_Save;

// Handles to some of the controls in window
global he_FastICA_suffix;

// The needed main variables
global g_FastICA_ica_sig;
global g_FastICA_ica_A;
global g_FastICA_ica_W;
global g_FastICA_white_sig;
global g_FastICA_white_wm;
global g_FastICA_white_dwm;
global g_FastICA_pca_E;
global g_FastICA_pca_D;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This should not take long...
//watchonInFigure = watchon;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
select action
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Save' then
 
 suffix = stripblanks(get(he_FastICA_suffix, 'String')); // The suffix for the variables

 printf('Saving results as global variable '+suffix+' in Scilab workspace.\n');
 printf('Do global '+suffix+' in Scilab workspace to read the variable.\n');
//  execstr('IC'+suffix+'='+(sci2exp(g_FastICA_ica_sig,0)));
//  execstr('A'+suffix+'='+(sci2exp(g_FastICA_ica_A,0)));
//  execstr('W'+suffix+'='+(sci2exp(g_FastICA_ica_W,0)));
//  execstr('whitesig'+suffix+'='+(sci2exp(g_FastICA_white_sig,0)));
//  execstr('whiteningMatrix'+suffix+'='+(sci2exp(g_FastICA_white_wm,0)));
//  execstr('dewhiteningMatrix'+suffix+'='+(sci2exp(g_FastICA_white_dwm,0)));
//  execstr('E'+suffix+'='+(sci2exp(g_FastICA_pca_E,0)));
//  execstr('D'+suffix+'='+(sci2exp(g_FastICA_pca_D,0)));
 execstr(suffix+'.IC'+'='+(sci2exp(g_FastICA_ica_sig,0)));
 execstr(suffix+'.A'+'='+(sci2exp(g_FastICA_ica_A,0)));
 execstr(suffix+'.W'+'='+(sci2exp(g_FastICA_ica_W,0)));
 execstr(suffix+'.whitesig'+'='+(sci2exp(g_FastICA_white_sig,0)));
 execstr(suffix+'.whiteningMatrix'+'='+(sci2exp(g_FastICA_white_wm,0)));
 execstr(suffix+'.dewhiteningMatrix'+'='+(sci2exp(g_FastICA_white_dwm,0)));
 execstr(suffix+'.E'+'='+(sci2exp(g_FastICA_pca_E,0)));
 execstr(suffix+'.D'+'='+(sci2exp(g_FastICA_pca_D,0)));

execstr("global "+suffix);
//  assignin('base',['IC' suffix],g_FastICA_ica_sig);
//  assignin('base',['A' suffix],g_FastICA_ica_A);
//  assignin('base',['W' suffix],g_FastICA_ica_W);
//  assignin('base',['whitesig' suffix],g_FastICA_white_sig);
//  assignin('base',['whiteningMatrix' suffix],g_FastICA_white_wm);
//  assignin('base',['dewhiteningMatrix' suffix],g_FastICA_white_dwm);
//  assignin('base',['E' suffix],g_FastICA_pca_E);
//  assignin('base',['D' suffix],g_FastICA_pca_D);

 close(hf_FastICA_Save);                  // close the dialog
 
 // Use return to avoid reaching the watchoff statement at the end
 // (There used to be a 'break' statement here, but it resulted in
 // errors in more recent version of Matlab -- jarmo)
 return;
  
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Cancel' then
 
 close(hf_FastICA_Save);                       // do nothing just exit

 // Use return to avoid reaching the watchoff statement at the end
 // (There used to be a 'break' statement here, but it resulted in
 // errors in more recent version of Matlab -- jarmo)
 return;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'Help' then

 fastica_gui_help('gui_sc');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
end    // switch

//watchoff (watchonInFigure);
endfunction


function fastica_gui_help(which_help)
//
// Used by FASTICAG

// All the help texts and title used by GUI are stored here. 
// Make changes here.
// Also displays the helpwindow with the selected text

// @(#)$Id: gui_help.m,v 1.6 2005/10/19 13:05:34 jarmo Exp $

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
select which_help
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'pcamat' then

  helptitle = 'FastICA: Reduce dimension';
  helptext=[ ...
    'You may reduce the dimension of the data by selecting only the     '
    'subspace corresponding to certain eigenvalues of the covariance    '
    'matrix of the data. Give the indices of the first and last         '
    'eigenvalues (sorted in descending order) to be included (all       '
    'eigenvalues in between will be included as well).  The eigenvalues '
    'and their indices can be seen in the graphical plot now on the     '
    'screen. The heights of the bars give the eigenvalues, with indices '
    'below.                                                             '
    '                                                                   '
    'For example, give ''1'' and ''n'' if you want to reduce the dimension  '
    'to n by principal component analysis, which means discarding the   '
    'subspaces corresponding to the smallest eigenvalues. Such a        '
    'dimension reduction may reduce noise and improve the performance of'
    'ICA.                                                               '];


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'gui_cb_about' then

  helptitle='About FastICA';
  helptext =[ ...
    'ICA Toolbox for Scilab 5                                                 '
    'Holger Nahrstaedt - 2011                                                  '
    '                                                                          '
    'October 19 2005                                              '
    'Copyright (c) Hugo Gvert, Jarmo Hurri, Jaakko Srel, and Aapo Hyvrinen.'
    '                                                                          '
    'For more information please see:                                          '
    'http://www.cis.hut.fi/projects/ica/fastica/                               '];

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'gui_cb_help' then

  helptitle='FastICA GUI';
  helptext = [...
    'Basic function:                                                            '
    '                                                                           '
    '- Click LOAD DATA and give the name of the variable that contains          '
    '  the data.                                                                '
    '                                                                           '
    '- Click DO ICA to perform the analysis.                                    '
    '                                                                           '
    '- Click SAVE RESULTS to store the results for future use.                  '
    '                                                                           '
    'Options:                                                                   '
    '                                                                           '
    'If the input matrix contains the signals as column vectors instead of      '
    'row vectors, click on TRANSPOSE to transpose the data matrix.              '
    '                                                                           '
    'Click on PLOT DATA to see the data as 1-D time signals.                    '
    '                                                                           '
    'Clicking REDUCE DIM gives you a graphical plot of the eigenvalue           '
    'structure of the covariance matrix of the data. You can then reduce        '
    'the dimension of the data by retaining only the subspaces corresponding to '
    'the largest (or smallest) eigenvalues (i.e. variances). To undo this       '
    'operation click ORIGINAL DIM. You can plot the whitened (preprocessed      '
    'data) by PLOT WHITENED.                                                    '
    '                                                                           '
    'Click on DO ICA to perform independent component analysis.                 '
    'Clicking on PLOT ICS has the same effect, except that DO ICA forces        '
    'recomputation of ICA.                                                      '
    '                                                                           '
    'You can choose the decorrelation approach by the ''Approach'' drop-down menu:'
    'deflation means that the independent components are estimated              '
    'one-by-one, whereas in the symmetric approach they are estimated in        '
    'parallel. You can now choose the number of independent components to be    '
    'estimated in both deflation and symmetric approaches.                      '
    '                                                                           '
    'You have a choice of three nonlinearities:                                 '
    '                                                                           '
    '''pow3'' (default) :  g(u)=u^3                                               '
    '''tanh''           :  g(u)=tanh(u)                                           '
    '''gauss''          :  g(u)=u*exp(-u^2/2)                                     '
    '''skew''           :  g(u)=u^2                                               '
    '                                                                           '
    'For example, you could choose approach=''symmetric'' and nonlinearity=''tanh'' '
    'to perform maximum likelihood ICA estimation for supergaussian data.       '
    '                                                                           '
    'If the algorithm does not seem to converge, you can use the stabilized     '
    'version of the fixed-point algorithm. To use the stabilized version,       '
    'choose ''on'' from the drop-down menu ''Stabilization''.                       '
    'If you have specified a value less than 1 for the parameter ''mu'' from      '
    'the ''Advanced Options'' menu then the ''Stabilization'' drop-down menu is     '
    'not active. This is because if the parameter ''mu'' is less than 1 then the  '
    'program will use the stabilized code. Please see the help for              '
    'Advanced Options for more information about stabilization.                 '
    '                                                                           '
    'The ADVANCED OPTIONS menu has its own HELP button.                         '
    '                                                                           '
    'During computations, an INTERRUPT button appears. Clicking the button      '
    'interrupts the computations.                                               '];
  
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'gui_advc' then

  helptitle='FastICA GUI: Advanced options';
  helptext = [...
    'Advanced options:                                                     '
    '                                                                      '
    'In some cases, it may be desired to improve the statistical           '
    'performance of the algorithm by using a fine-tuning procedure. This   '
    'means that after (initial) convergence, the algorithm is run          '
    '(possibly) with a different nonlinearity, and using a smaller step    '
    'size and the stabilized version of the fixed-point algorithm.         '
    'You can specify the nonlinearity that will then be used with the      '
    'parameter ''Finetune''. If you set the the finetuning to ''off'', then the'
    'fine-tuning won''t be done.                                            '
    '                                                                      '
    'You can also fine-tune the nonlinearities used in the fixed-point     '
    'algorithm.                                                            ' 
    'The nonlinearities tanh and gauss contain parameters a1 and a2, so    '
    'that the nonlinearities are in fact defined as:                       '
    '''tanh''           :  g(u)=tanh(a1*u)                                   '
    '''gauss''          :  g(u)=u*exp(-a2*u^2/2)                             '
    'The default values of a1 and a2 are 1, in which case they effectively '
    'disappear from the definitions.                                       '
    '                                                                      '
    'If the algorithm does not seem to converge, you can use the stabilized'
    'version of the fixed-point algorithm. There are two ways of doing     '
    'this. The first one is to explicitly specify the value of the step    '
    'size parameter ''mu''. The default value is 1. Choosing a value that    '
    'is smaller than 1 implies that the computations are made using the    '
    'stabilized fixed-point algorithm. The second way to use the stabilized'
    'version is simpler: choose ''on'' in the drop-down menu ''stabilization'' '
    '(on the main menu page). Then the value of mu will be changed         '
    'automatically during the ICA calculations. If the program senses that '
    'the algorithm is stuck between two points, it will halve the value of '
    'mu (.5 * mu) for duration of one round. (This is called a ''stroke.''). '
    'Also if there is no convergence before half of the maximum number of  '
    'iterations has been reached then the mu will be halved for the rest   ' 
    'of the rounds.                                                        '
    '                                                                      '
    'The parameter ''epsilon'' is used to decide if the algorithm has        '
    'converged. A larger epsilon makes the convergence test less strict.   '
    'Note that if you use finetuning or stabilization, epsilon may need to '
    'be reduced accordingly.                                               '
    '                                                                      '
    '''Maximum number of iterations'' gives the absolute maximum of          '
    'iterations used in the estimation procedure. In the deflation         '
    'approach, this is iterations per component.                           '
    '                                                                      '
    'You can input the ''Initial state'' of the algorithm, i.e. the initial  '
    'value for A. Choose ''guess'' in the drop-down menu ''Initial state'',    '
    'click on ''Load Initial guess'', and give the name of the variable in   '
    'Matlab workspace that contains the initial value.                     '
    '                                                                      '
    'In the drop-down menu ''Display mode'' you can choose if the results are'
    'plotted during computations.  You may wish to switch this off         '
    'especially if you have lots of data which takes a long time to plot.  '
    '''Iteration between displays'' tells how often the running estimates of '
    'the independent components are plotted: A value of 1 means after every'
    'iteration.                                                            '
    '                                                                      '
    'If the data vector is very long (more than 10 000 points), it may be  '
    'advisable to use only a part of the data at every iteration. The      '
    'option ''Sample size'' allows you to give the proportion (0-1) of the   '
    'data that is used at every step. The sample is chosen randomly at     '
    'every step.                                                           '
    '                                                                      '
    'Click on DEFAULT to return to default values for all advanced options.'
    'You can make the new values take effect without closing the window by '
    'clicking APPLY.                                                       '];

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'gui_lc_data' then

  helptitle='FastICA GUI: Load data';
  helptext = [...
    'Input the name of the variable in Matlab workspace that contains the  '
    'data. The data must be in a single matrix, each row (or column) giving'
    'the values of one signal. If the signals are in column vectors, click '
    'TRANSPOSE after loading the data to transpose the data matrix.        '
    '                                                                      '
    'If the data is in a file, load it to Matlab workspace first.          '];

case 'gui_lc_guess'

  helptitle='FastICA GUI: Load guess';
  helptext = [...
    'Input the name of the variable in Matlab workspace that contains the'
    'initial value for the mixing matrix A, and click OK. If the initial '
    'value is in a file, load it to Matlab workspace first.              '];

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
case 'gui_sc' then

  helptitle='FastICA GUI: Save results';
  helptext = [...
    'The results will be saved as variables in Matlab workspace.            '
    'You give a suffix that identifies the variables. For example, if you   '
    'give ''_FASTICA'', the results will be stored in the following variables:'
    '                                                                       '
    'W_FASTICA   : estimate of the separating matrix                        '
    'A_FASTICA   : estimate of the mixing matrix                            '
    'IC_FASTICA  : estimated independent components (row vectors)           '
    '                                                                       '
    'Additional results related to preprocessing:                           '
    'D_FASTICA and E_FASTICA    : give the eigenvalue decomposition of the  '
    '                             covariance matrix                         '
    'whiteningMatrix_FASTICA    : matrix performing whitening and dimension '
    '                             reduction                                 '
    'dewhiteningMatrix_FASTICA  : the pseudoinverse of the whitening matrix '
    'whitesig_FASTICA           : whitened (i.e. preprocessed) signals.     '];

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

end

//helpwin(helptext, helptitle);

messagebox(helptext,helptitle);


endfunction

